
! Copyright (C) 2024 Eddie Harris-Lee, J. K. Dewhurst and S. Sharma.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.

subroutine energyafsp
use modmain
use modmpi
use modomp
implicit none
! local variables
integer ik,ist,i,j,l,nthd
real(8) ca,sm,v(3),wo
complex(8) zm(2,2)
! allocatable arrays
complex(8), allocatable :: apwalm(:,:,:,:),evecfv(:,:),evecsv(:,:)
complex(8), allocatable :: wfmt(:,:,:,:),wfgk(:,:,:)
complex(8), allocatable :: pmat(:,:,:),smat(:,:,:,:),h(:,:)
! coupling constant of the external spin-polarised A-field (-1/c)
ca=-1.d0/solsc
sm=0.d0
! begin parallel loop over k-points
call holdthd(nkpt/np_mpi,nthd)
!$OMP PARALLEL DEFAULT(SHARED) &
!$OMP PRIVATE(apwalm,evecfv,evecsv) &
!$OMP PRIVATE(wfmt,wfgk,pmat,smat,h) &
!$OMP PRIVATE(i,j,l,v,zm,ist,wo) &
!$OMP REDUCTION(+:sm) &
!$OMP NUM_THREADS(nthd)
allocate(apwalm(ngkmax,apwordmax,lmmaxapw,natmtot))
allocate(evecfv(nmatmax,nstfv),evecsv(nstsv,nstsv))
allocate(wfmt(npcmtmax,natmtot,nspinor,nstsv),wfgk(ngkmax,nspinor,nstsv))
allocate(pmat(nstsv,nstsv,3),smat(nstsv,nstsv,2,2),h(nstsv,nstsv))
!$OMP DO
do ik=1,nkpt
! distribute among MPI processes
  if (mod(ik-1,np_mpi) /= lp_mpi) cycle
! get the eigenvectors from file
  call getevecfv(filext,ik,vkl(:,ik),vgkl(:,:,:,ik),evecfv)
  call getevecsv(filext,ik,vkl(:,ik),evecsv)
! find the matching coefficients
  call match(ngk(1,ik),vgkc(:,:,1,ik),gkc(:,1,ik),sfacgk(:,:,1,ik),apwalm)
! calculate the wavefunctions for all states
  call genwfsv(.true.,.true.,nstsv,[0],ngridg,igfft,ngk(:,ik),igkig(:,:,ik), &
   apwalm,evecfv,evecsv,wfmt,ngkmax,wfgk)
! calculate the momentum matrix elements in the second-variational basis
  call genpmatk(ngk(:,ik),igkig(:,:,ik),vgkc(:,:,:,ik),wfmt,wfgk,pmat)
! generate the spin operator matrix elements in the second-variational basis
  call gensmatk(evecsv,smat)
! multiply the momentum and spin matrices to form the Hamiltonian term
  h(:,:)=0.d0
  do l=1,3
    v(:)=ca*afspc(l,:)
    zm(1,1)=v(3)
    zm(1,2)=cmplx(v(1),-v(2),8)
    zm(2,1)=cmplx(v(1),v(2),8)
    zm(2,2)=-v(3)
    do j=1,2
      do i=1,2
        call zhemm('L','U',nstsv,nstsv,zm(i,j),pmat(:,:,l),nstsv,smat(:,:,i,j),&
         nstsv,zone,h,nstsv)
      end do
    end do
  end do
! add to the expectation value
  do ist=1,nstsv
    wo=occsv(ist,ik)
    if (abs(wo) < epsocc) cycle
    wo=wo*wkpt(ik)
    sm=sm+wo*dble(h(ist,ist))
  end do
end do
!$OMP END DO
deallocate(apwalm,evecfv,evecsv)
deallocate(wfmt,wfgk,pmat,smat,h)
!$OMP END PARALLEL
call freethd(nthd)
! add sums from each process and redistribute
if (np_mpi > 1) then
  call mpi_allreduce(mpi_in_place,sm,1,mpi_double_precision,mpi_sum,mpicom, &
   ierror)
end if
! subtract from kinetic energy
engykn=engykn-0.5d0*sm
end subroutine

